varying vec2 LightmapCoords;
varying vec2 texcoord;
varying vec4 glcolor;
attribute vec4 mc_Entity;
attribute vec2 mc_midTexCoord;
flat varying float material;
varying vec3 MixedLights;
flat varying vec3 Normal;

vec3 ViewPos;
#include "/global/light_colors.vsh"

void init_generic() {
    init_colors();

    gl_Position = ftransform();
    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    LightmapCoords = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    LightmapCoords = (LightmapCoords * 33.05f / 32.0f) - (1.05f / 32.0f);
    material = mc_Entity.x;

    ViewPos = (gl_ModelViewMatrix * gl_Vertex).xyz;

    Normal = normalize(gl_NormalMatrix * gl_Normal);
    vec3 NormalA;
    if(material == 10001 || material == 10004 || material == 10005 || material == 10006) {
        NormalA = normalize(upPosition);
        // Make grass darker at the bottom. It looks better this way
        if(gl_MultiTexCoord0.t > mc_midTexCoord.t && (material == 10004 || material == 10005)) NormalA *= 0.5;
    }
    else {
        NormalA = Normal;
    }
    glcolor = gl_Color;
    float Dist = length(ViewPos);
    
    float NdotL;
    #ifndef DIMENSION_NETHER
    NdotL = max(dot(NormalA, sunOrMoonPosN), 0);
    #endif

    #ifdef HANDHELD_LIGHTS
    float HandheldLight = max(float(heldBlockLightValue), heldBlockLightValue2);
    LightmapCoords.x = max(LightmapCoords.x, (HandheldLight-Dist)/15.0);
    #endif

    const vec3 TorchColor = to_linear(vec3(LM_RED, LM_GREEN, LM_BLUE));
    vec3 MinLight = to_linear(vec3(MIN_LIGHT_AMOUNT));
    MinLight += nightVision/3;
     
    #ifndef DIMENSION_OVERWORLD
        LightmapCoords.y = 1;
    #endif

    #ifdef DIMENSION_OVERWORLD
        float TorchNerfFactor = min(get_luminance(SUN_AMBIENT)*2, 1);

    #elif defined DIMENSION_END
        float TorchNerfFactor = 0.66;
    #else
        float TorchNerfFactor = 0;
    #endif

    #ifndef DIMENSION_NETHER
    // This cuts off direct sunlight it semi-occulded areas.
    float FakeShadowFactor = smoothstep(0.85, 0.96, LightmapCoords.y);

    SUN_AMBIENT += SUN_DIRECT * NdotL * FakeShadowFactor;
    #endif 

    LightmapCoords.x *= LightmapCoords.x;

    LightmapCoords.x = mix(LightmapCoords.x, LightmapCoords.x*(1-TorchNerfFactor), LightmapCoords.y);
    MixedLights = TorchColor * LightmapCoords.x + SUN_AMBIENT * LightmapCoords.y;
    MixedLights = mix(MinLight, MixedLights, max(LightmapCoords.x, LightmapCoords.y));
    MixedLights *= 1-darknessLightFactor;
}
